@@ -1,4 +1,7 @@
 #include <Rcpp.h>
+#include <cmath>
+#include <iostream>
+#include <unistd.h>
 using namespace Rcpp;
 
 
@@ -87,3 +90,50 @@ NumericMatrix schwartzberg(IntegerVector from,
 
   return ret;
 }
+
+// [[Rcpp::export(rng = false)]]
+NumericMatrix bbox_reock(IntegerMatrix dm,
+                         NumericVector areas,
+                         NumericMatrix extents,
+                         const int nd) {
+  const int n = dm.nrow();
+  const int n_plans = dm.ncol();
+
+  NumericMatrix out(nd, n_plans);
+  std::vector<double> dist_area(nd);
+
+  std::vector<double> min_xmin_dist(nd);
+  std::vector<double> max_xmax_dist(nd);
+  std::vector<double> min_ymin_dist(nd);
+  std::vector<double> max_ymax_dist(nd);
+  for (int c = 0; c < dm.ncol(); c++) {
+    // Initialize aggregation vectors
+    for (int d = 0; d < nd; d++) {
+      dist_area[d] = 0.0;
+      min_xmin_dist[d] = INFINITY;
+      max_xmax_dist[d] = -INFINITY;
+      min_ymin_dist[d] = INFINITY;
+      max_ymax_dist[d] = -INFINITY;
+    }
+
+    // Aggregate by district
+    for (int r = 0; r < n; r++) {
+      int dist = dm(r, c) - 1; // Convert to 0-indexed
+
+      dist_area[dist] += areas(r);
+      if (extents(r, 0) < min_xmin_dist[dist]) min_xmin_dist[dist] = extents(r, 0);
+      if (extents(r, 1) < min_ymin_dist[dist]) min_ymin_dist[dist] = extents(r, 1);
+      if (extents(r, 2) > max_xmax_dist[dist]) max_xmax_dist[dist] = extents(r, 2);
+      if (extents(r, 3) > max_ymax_dist[dist]) max_ymax_dist[dist] = extents(r, 3);
+    }
+
+    // Compute compactness for each district
+    for (int d = 0; d < nd; d++) {
+      double mbbox = (max_xmax_dist[d] - min_xmin_dist[d]) *
+        (max_ymax_dist[d] - min_ymin_dist[d]);
+      out(d, c) = dist_area[d] / mbbox;
+    }
+  }
+
+  return out;
+}
